#!/usr/bin/env ts-node
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
// tslint:disable:no-shadowed-variable
import test from 'blue-tape';
import sinon from 'sinon';
import StateSwitch from './state-switch';
test('on()', (t) => __awaiter(this, void 0, void 0, function* () {
    const ss = new StateSwitch();
    t.notOk(ss.on(), 'default is not on');
    ss.on('pending');
    t.equal(ss.on(), 'pending', 'should be state pending');
    ss.on(true);
    t.equal(ss.on(), true, 'should be state true');
    t.notOk(ss.off(), 'should not off');
    ss.off(true);
    t.notOk(ss.on(), 'should not ON after off()');
}));
test('off()', (t) => __awaiter(this, void 0, void 0, function* () {
    const ss = new StateSwitch();
    t.ok(ss.off(), 'default is off');
    t.equal(ss.off(), true, 'should in state true');
    ss.off('pending');
    t.equal(ss.off(), 'pending', 'should be state pending');
    ss.off(true);
    t.equal(ss.off(), true, 'should be state true');
    t.notOk(ss.on(), 'should not on');
    ss.on(true);
    t.notOk(ss.off(), 'should not OFF after on()');
}));
test('pending()', (t) => __awaiter(this, void 0, void 0, function* () {
    const ss = new StateSwitch();
    t.notOk(ss.pending(), 'default is not pending');
    ss.on('pending');
    t.ok(ss.pending(), 'should in pending state');
    ss.on(true);
    t.notOk(ss.pending(), 'should not in pending state');
    ss.off('pending');
    t.ok(ss.pending(), 'should in pending state');
}));
test('name', (t) => __awaiter(this, void 0, void 0, function* () {
    const CLIENT_NAME = 'StateSwitchTest';
    const ss = new StateSwitch(CLIENT_NAME);
    t.is(ss.name(), CLIENT_NAME, 'should get the same client name as init');
}));
test('version()', t => {
    const ss = new StateSwitch();
    t.ok(ss.version(), 'should get version');
    t.end();
});
test('ready()', (t) => __awaiter(this, void 0, void 0, function* () {
    const spy = sinon.spy();
    const ss = new StateSwitch();
    ss.ready('off').then(() => spy('off')).catch(() => t.fail('rejection'));
    yield new Promise(resolve => setImmediate(resolve));
    t.equal(spy.callCount, 1, 'should be read off at the initial state');
    spy.resetHistory();
    ss.ready('on', true).catch(() => spy('on'));
    yield new Promise(resolve => setImmediate(resolve));
    t.equal(spy.callCount, 1, 'should catch the exception when noCross=true');
    spy.resetHistory();
    ss.ready('on').then(() => spy('on')).catch(() => t.fail('rejection'));
    ss.on(true);
    yield new Promise(resolve => setImmediate(resolve));
    t.equal(spy.callCount, 1, 'should ready(on)');
    spy.resetHistory();
    ss.ready('on').then(() => spy('on')).catch(() => t.fail('rejection'));
    yield new Promise(resolve => setImmediate(resolve));
    t.equal(spy.callCount, 1, 'should ready(on) when already on');
    spy.resetHistory();
    ss.ready('off').then(() => spy('off')).catch(() => t.fail('rejection'));
    yield new Promise(resolve => setImmediate(resolve));
    t.equal(spy.callCount, 0, 'should not ready(off) when its on');
    ss.off(true);
    yield new Promise(resolve => setImmediate(resolve));
    t.equal(spy.callCount, 1, 'should ready(off) after call off(true)');
}));
//# sourceMappingURL=state-switch.spec.js.map